/*
Copyright 2020 Appboxo pte. ltd.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package com.appboxo.appboxo_sdk

import android.app.Activity
import androidx.annotation.NonNull
import com.appboxo.js.params.CustomEvent
import com.appboxo.sdk.Appboxo
import com.appboxo.sdk.Config
import com.appboxo.sdk.Miniapp
import com.appboxo.sdk.MiniappConfig
import com.appboxo.ui.main.AppboxoActivity
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar

class AppboxoSdkPlugin(private val activity: Activity) : MethodCallHandler {

    private val lifecycleListener = object : Miniapp.LifecycleListener {
        override fun onClose(miniapp: Miniapp) {
            lifecycleEventsStreamHandler.events?.success(
                    mapOf(
                            "appId" to miniapp.appId,
                            "lifecycle" to "onClose"
                    )
            )
        }

        override fun onError(miniapp: Miniapp, message: String) {
            lifecycleEventsStreamHandler.events?.success(
                    mapOf(
                            "appId" to miniapp.appId,
                            "lifecycle" to "onError",
                            "error" to message
                    )
            )
        }

        override fun onLaunch(miniapp: Miniapp) {
            lifecycleEventsStreamHandler.events?.success(
                    mapOf(
                            "appId" to miniapp.appId,
                            "lifecycle" to "onLaunch"
                    )
            )
        }

        override fun onPause(miniapp: Miniapp) {
            lifecycleEventsStreamHandler.events?.success(
                    mapOf(
                            "appId" to miniapp.appId,
                            "lifecycle" to "onPause"
                    )
            )
        }

        override fun onResume(miniapp: Miniapp) {
            lifecycleEventsStreamHandler.events?.success(
                    mapOf(
                            "appId" to miniapp.appId,
                            "lifecycle" to "onResume"
                    )
            )
        }
    }
    private val customEventListener = object : Miniapp.CustomEventListener {
        override fun handle(miniAppActivity: AppboxoActivity, miniapp: Miniapp, customEvent: CustomEvent) {
            customEventsStreamHandler.events?.success(
                    mapOf("appId" to miniapp.appId,
                            "requestId" to customEvent.requestId,
                            "type" to customEvent.type,
                            "errorType" to customEvent.errorType,
                            "payload" to customEvent.payload))
        }
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "setConfig" -> {
                val clientId = call.argument<String>("client_id")!!
                val multitaskMode = call.argument<Boolean>("multitask_mode") ?: true
                val theme = when (call.argument<String>("theme")) {
                    "light" -> Config.Theme.LIGHT
                    "dark" -> Config.Theme.DARK
                    else -> Config.Theme.SYSTEM
                }

                Appboxo.setConfig(Config.Builder()
                        .setClientId(clientId)
                        .multitaskMode(multitaskMode)
                        .setTheme(theme)
                        .build())
            }
            "openMiniApp" -> {
                val appId = call.argument<String>("app_id")!!
                val payload = call.argument<String>("payload") ?: ""
                val data = call.argument<Map<String, Any>>("data")
                val theme = when (call.argument<String>("theme")) {
                    "light" -> Config.Theme.LIGHT
                    "dark" -> Config.Theme.DARK
                    "system" -> Config.Theme.SYSTEM
                    else -> null
                }
                Appboxo.getMiniapp(appId, payload, null, data)
                        .setCustomEventListener(customEventListener)
                        .setLifecycleListener(lifecycleListener)
                        .apply {
                            if (theme != null)
                                setConfig(MiniappConfig.Builder()
                                        .setTheme(theme)
                                        .build())
                        }
                        .open(activity)
            }
            "hideAllMiniApps" -> {
                Appboxo.hideMiniapps()
            }

            "sendEvent" -> {
                try {
                    val map = call.argument<Map<String, *>>("custom_event")!!
                    val appId = map["appId"] as String

                    @Suppress("UNCHECKED_CAST")
                    val customEvent = CustomEvent(requestId = map["requestId"] as? String,
                            type = map["type"] as String,
                            errorType = map["errorType"] as? String,
                            payload = map["payload"] as Map<String, Any>
                    )
                    Appboxo.getMiniapp(appId)?.sendEvent(customEvent)
                } catch (e: Exception) {
                    print(e)
                }
            }
            "logout" -> {
                Appboxo.logout()
            }
        }
    }


    companion object {
        private val customEventsStreamHandler = CustomEventsStreamHandler()
        private val lifecycleEventsStreamHandler = LifecycleEventsStreamHandler()

        @JvmStatic
        fun registerWith(registrar: Registrar) {
            Appboxo.init(registrar.activity().application)
            val channel = MethodChannel(registrar.messenger(), "appboxo_sdk")
            val customEventChannel = EventChannel(registrar.messenger(), "appboxo_sdk_custom_events")
            val lifecycleChannel = EventChannel(registrar.messenger(), "appboxo_sdk_lifecycle")
            val plugin = AppboxoSdkPlugin(registrar.activity())
            channel.setMethodCallHandler(plugin)
            lifecycleChannel.setStreamHandler(lifecycleEventsStreamHandler)
            customEventChannel.setStreamHandler(customEventsStreamHandler)
        }
    }
}
